@@ -425,15 +425,14 @@ tcp_connect(getdns_upstream *upstream, getdns_transport_list_t transport)
 	endpoints.sae_srcaddrlen = 0;
 	endpoints.sae_dstaddr = (struct sockaddr *)&upstream->addr;
 	endpoints.sae_dstaddrlen = upstream->addr_len;
-	if (connectx(fd, &endpoints, SAE_ASSOCID_ANY,  
+	if (connectx(fd, &endpoints, SAE_ASSOCID_ANY,
 	             CONNECT_DATA_IDEMPOTENT | CONNECT_RESUME_ON_READ_WRITE,
-	             NULL, 0, NULL, NULL) == -1) {
-		if (errno != EINPROGRESS) {
-			close(fd);
-			return -1;
-		}
+	             NULL, 0, NULL, NULL) == 0) {
+		return fd;
+	}
+	if (errno == EINPROGRESS) {
+		return fd;
 	}
-	return fd;
 #else
 	(void)transport;
 #endif
@@ -588,6 +587,7 @@ _getdns_cancel_stub_request(getdns_network_req *netreq)
 #else
 		close(netreq->fd);
 #endif
+		netreq->fd = -1;
 	}
 }
 
@@ -606,13 +606,13 @@ stub_timeout_cb(void *userarg)
 #else
 		close(netreq->fd);
 #endif
+		netreq->fd = -1;
 		netreq->upstream->udp_timeouts++;
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
 		if (netreq->upstream->udp_timeouts % 100 == 0)
-			DEBUG_DAEMON("%s %-40s : Upstream stats: Transport=UDP - Resp=%d,Timeouts=%d\n",
-			             STUB_DEBUG_DAEMON, netreq->upstream->addr_str,
+			_getdns_upstream_log(netreq->upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+			    "%-40s : Upstream stats: Transport=UDP - Resp=%d,Timeouts=%d\n",
+			             netreq->upstream->addr_str,
 			             (int)netreq->upstream->udp_responses, (int)netreq->upstream->udp_timeouts);
-#endif
 		stub_next_upstream(netreq);
 	} else {
 		netreq->upstream->responses_timeouts++;
@@ -908,12 +908,11 @@ tls_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 	            STUB_DEBUG_SETUP_TLS, __FUNC__, upstream->fd, err,
 	            X509_verify_cert_error_string(err));
 #endif
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
 	if (!preverify_ok && !upstream->tls_fallback_ok)
-			DEBUG_DAEMON("%s %-40s : Verify failed : Transport=TLS - *Failure* -  (%d) \"%s\"\n",
-		                  STUB_DEBUG_DAEMON, upstream->addr_str, err,
-			              X509_verify_cert_error_string(err));
-#endif
+		_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+		    "%-40s : Verify failed : Transport=TLS - *Failure* -  (%d) \"%s\"\n",
+		    upstream->addr_str, err,
+		    X509_verify_cert_error_string(err));
 
 	/* First deal with the hostname authentication done by OpenSSL. */
 #ifdef X509_V_ERR_HOSTNAME_MISMATCH
@@ -945,11 +944,10 @@ tls_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 		if (upstream->tls_fallback_ok)
 			DEBUG_STUB("%s %-35s: FD:  %d, WARNING: Proceeding even though pinset validation failed!\n",
 			            STUB_DEBUG_SETUP_TLS, __FUNC__, upstream->fd);
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
 		else
-			DEBUG_DAEMON("%s %-40s : Conn failed   : Transport=TLS - *Failure* - Pinset validation failure\n",
-		                  STUB_DEBUG_DAEMON, upstream->addr_str);
-#endif
+			_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+			    "%-40s : Conn failed   : Transport=TLS - *Failure* - Pinset validation failure\n",
+			    upstream->addr_str);
 	} else {
 		/* If we _only_ had a pinset and it is good then force succesful
 		   authentication when the cert self-signed
@@ -960,10 +958,9 @@ tls_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 			preverify_ok = 1;
 			DEBUG_STUB("%s %-35s: FD:  %d, Allowing self-signed (%d) cert since pins match\n",
 		           STUB_DEBUG_SETUP_TLS, __FUNC__, upstream->fd, err);
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
-			DEBUG_DAEMON("%s %-40s : Verify passed : Transport=TLS - Allowing self-signed cert since pins match\n",
-		                  STUB_DEBUG_DAEMON, upstream->addr_str);
-#endif
+			_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG, 
+			    "%-40s : Verify passed : Transport=TLS - Allowing self-signed cert since pins match\n",
+			    upstream->addr_str);
 		}
 	}
 
@@ -1417,6 +1414,7 @@ stub_udp_read_cb(void *userarg)
 #else
 			close(netreq->fd);
 #endif
+			netreq->fd = -1;
 			stub_next_upstream(netreq);
 		}
 		netreq->debug_end_time = _getdns_get_time_as_uintt64();
@@ -1439,8 +1437,8 @@ stub_udp_read_cb(void *userarg)
 	closesocket(netreq->fd);
 #else
 	close(netreq->fd);
-	netreq->fd = -1;
 #endif
+	netreq->fd = -1;
 	while (GLDNS_TC_WIRE(netreq->response)) {
 		DEBUG_STUB("%s %-35s: MSG: %p TC bit set in response \n", STUB_DEBUG_READ, 
 		             __FUNC__, (void*)netreq);
@@ -1474,13 +1472,12 @@ stub_udp_read_cb(void *userarg)
 	netreq->debug_end_time = _getdns_get_time_as_uintt64();
 	_getdns_netreq_change_state(netreq, NET_REQ_FINISHED);
 	upstream->udp_responses++;
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
 	if (upstream->udp_responses == 1 || 
 	    upstream->udp_responses % 100 == 0)
-		DEBUG_DAEMON("%s %-40s : Upstream stats: Transport=UDP - Resp=%d,Timeouts=%d\n",
-		             STUB_DEBUG_DAEMON, upstream->addr_str,
-		             (int)upstream->udp_responses, (int)upstream->udp_timeouts);
-#endif
+		_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+		    "%-40s : Upstream stats: Transport=UDP - Resp=%d,Timeouts=%d\n",
+		    upstream->addr_str,
+		    (int)upstream->udp_responses, (int)upstream->udp_timeouts);
 	_getdns_check_dns_req_complete(dnsreq);
 }
 
@@ -1538,6 +1535,7 @@ stub_udp_write_cb(void *userarg)
 #else
 			close(netreq->fd);
 #endif
+			netreq->fd = -1;
 			stub_next_upstream(netreq);
 		}
 		netreq->debug_end_time = _getdns_get_time_as_uintt64();
@@ -1734,11 +1732,10 @@ upstream_write_cb(void *userarg)
 	case STUB_NO_AUTH:
 		/* Cleaning up after connection or auth check failure. Need to fallback. */
 		stub_cleanup(netreq);
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
-		DEBUG_DAEMON("%s %-40s : Conn closed   : Transport=%s - *Failure*\n",
-		             STUB_DEBUG_DAEMON, upstream->addr_str,
-		             (upstream->transport == GETDNS_TRANSPORT_TLS ? "TLS" : "TCP"));
-#endif
+		_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+		    "%-40s : Conn closed   : Transport=%s - *Failure*\n",
+		    upstream->addr_str,
+		    (upstream->transport == GETDNS_TRANSPORT_TLS ? "TLS" : "TCP"));
 		if (fallback_on_write(netreq) == STUB_TCP_ERROR) {
 			/* TODO: Need new state to report transport unavailable*/
 			_getdns_netreq_change_state(netreq, NET_REQ_ERRORED);
@@ -1890,10 +1887,9 @@ upstream_select_stateful(getdns_network_req *netreq, getdns_transport_list_t tra
 		if (upstreams->upstreams[i].conn_state == GETDNS_CONN_BACKOFF &&
 		    upstreams->upstreams[i].conn_retry_time < now) {
 			upstreams->upstreams[i].conn_state = GETDNS_CONN_CLOSED;
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
-			DEBUG_DAEMON("%s %-40s : Re-instating upstream\n",
-		            STUB_DEBUG_DAEMON, upstreams->upstreams[i].addr_str);
-#endif
+			_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+			    "%-40s : Re-instating upstream\n",
+		            upstreams->upstreams[i].addr_str);
 		}
 	}
 
@@ -1965,7 +1961,7 @@ upstream_select(getdns_network_req *netreq)
 			return &upstreams->upstreams[i];
 		}
 		i+=GETDNS_UPSTREAM_TRANSPORTS;
-		if (i > upstreams->count)
+		if (i >= upstreams->count)
 			i = 0;
 	} while (i != upstreams->current_udp);
 
@@ -2024,11 +2020,10 @@ upstream_connect(getdns_upstream *upstream, getdns_transport_list_t transport,
 			upstream->tls_hs_state = GETDNS_HS_WRITE;
 		}
 		upstream->conn_state = GETDNS_CONN_SETUP;
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
-	DEBUG_DAEMON("%s %-40s : Conn init     : Transport=%s - Profile=%s\n", STUB_DEBUG_DAEMON, 
-	             upstream->addr_str, transport == GETDNS_TRANSPORT_TLS ? "TLS":"TCP",
-	             dnsreq->context->tls_auth_min == GETDNS_AUTHENTICATION_NONE ? "Opportunistic":"Strict");
-#endif
+		_getdns_upstream_log(upstream, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+		    "%-40s : Conn init     : Transport=%s - Profile=%s\n", 
+		    upstream->addr_str, transport == GETDNS_TRANSPORT_TLS ? "TLS":"TCP",
+		dnsreq->context->tls_auth_min == GETDNS_AUTHENTICATION_NONE ? "Opportunistic":"Strict");
 		break;
 	default:
 		return -1;
@@ -2072,8 +2067,10 @@ upstream_find_for_netreq(getdns_network_req *netreq)
 {
 	int fd = -1;
 	getdns_upstream *upstream;
-	for (size_t i = netreq->transport_current; 
-	            i < netreq->transport_count; i++) {
+	size_t i;
+
+	for (i = netreq->transport_current; 
+	     i < netreq->transport_count; i++) {
 		upstream = upstream_find_for_transport(netreq,
 		                                  netreq->transports[i],
 		                                  &fd);
@@ -2094,10 +2091,8 @@ upstream_find_for_netreq(getdns_network_req *netreq)
 	}
 	/* Handle better, will give generic error*/
 	DEBUG_STUB("%s %-35s: MSG: %p No valid upstream! \n", STUB_DEBUG_SCHEDULE, __FUNC__, (void*)netreq);
-#if defined(DAEMON_DEBUG) && DAEMON_DEBUG
-	DEBUG_DAEMON("%s *FAILURE* no valid transports or upstreams available!\n",
-	              STUB_DEBUG_DAEMON);
-#endif
+	_getdns_context_log(netreq->owner->context, GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
+	    "*FAILURE* no valid transports or upstreams available!\n");
 	return -1;
 }
 
