@@ -57,15 +57,21 @@ typedef struct getdns_item {
 	getdns_union     data;
 } getdns_item;
 
+typedef enum getdns_auth_state {
+	GETDNS_AUTH_NONE,      /* Not tried (Oppotunistic)*/
+	GETDNS_AUTH_FAILED,    /* Tried but failed or not possible*/
+	GETDNS_AUTH_OK,        /* Tried and worked (Strict) */
+} getdns_auth_state_t;
+
+#define GETDNS_STR_AUTH_NONE "None"
+#define GETDNS_STR_AUTH_FAILED "Failed"
+#define GETDNS_STR_AUTH_OK "Success"
 
 struct getdns_context;
 struct getdns_upstreams;
 struct getdns_upstream;
 
-/**
- * \defgroup strings String Constants
- * @{
- */
+/* String constants for internal use */
 #define GETDNS_STR_IPV4 "IPv4"
 #define GETDNS_STR_IPV6 "IPv6"
 #define GETDNS_STR_ADDRESS_TYPE "address_type"
@@ -113,14 +119,13 @@ struct getdns_upstream;
 #define GETDNS_STR_KEY_NSCOUNT "nscount"
 #define GETDNS_STR_KEY_ARCOUNT "arcount"
 
-#define TIMEOUT_FOREVER ((int64_t)-1)
+#define TIMEOUT_FOREVER ((uint64_t)0xFFFFFFFFFFFFFFFF)
 #define ASSERT_UNREACHABLE 0
 
 #define GETDNS_TRANSPORTS_MAX 3
 #define GETDNS_UPSTREAM_TRANSPORTS 2
+#define GETDNS_TRANSPORT_FAIL_MULT 5
 
-/** @}
- */
 
 /* declarations */
 struct getdns_dns_req;
@@ -154,21 +159,14 @@ priv_getdns_context_mf(getdns_context *context);
 
 typedef enum network_req_state_enum
 {
-	NET_REQ_NOT_SENT,
-	NET_REQ_IN_FLIGHT,
-	NET_REQ_FINISHED,
-	NET_REQ_CANCELED
+	NET_REQ_NOT_SENT  =  0,
+	NET_REQ_IN_FLIGHT =  1,
+	NET_REQ_FINISHED  =  2, /* Finish type in bits 2 and 3 */
+	NET_REQ_CANCELED  =  6, /* 2 + (1 << 2) */
+	NET_REQ_TIMED_OUT = 10, /* 2 + (2 << 2) */
+	NET_REQ_ERRORED   = 14  /* 2 + (3 << 2) */
 } network_req_state;
 
-/**
- * structure used by validate_extensions() to check extension formats
- */
-typedef struct getdns_extension_format
-{
-	char *extstring;
-	getdns_data_type exttype;
-} getdns_extension_format;
-
 
 /* State for async tcp stub resolving */
 typedef struct getdns_tcp_state {
@@ -176,7 +174,6 @@ typedef struct getdns_tcp_state {
 	uint8_t *write_buf;
 	size_t   write_buf_len;
 	size_t   written;
-	int      write_error;
 
 	uint8_t *read_buf;
 	size_t   read_buf_len;
@@ -185,7 +182,6 @@ typedef struct getdns_tcp_state {
 
 } getdns_tcp_state;
 
-
 /**
  * Request data
  **/
@@ -193,6 +189,12 @@ typedef struct getdns_network_req
 {
 	/* For storage in upstream->netreq_by_query_id */
 	_getdns_rbnode_t node;
+#ifdef HAVE_MDNS_SUPPORT
+	/*
+	 * for storage of continuous query context in hash table of cached results. 
+	 */
+	struct getdns_network_req * mdns_netreq_next;
+#endif /* HAVE_MDNS_SUPPORT */
 	/* the async_id from unbound */
 	int unbound_id;
 	/* state var */
@@ -214,6 +216,8 @@ typedef struct getdns_network_req
 	 */
 	int tsig_status;
 
+	getdns_redirects_t      follow_redirects;
+
 	/* For stub resolving */
 	struct getdns_upstream *upstream;
 	int                     fd;
@@ -222,8 +226,6 @@ typedef struct getdns_network_req
 	size_t                  transport_current;
 	getdns_tls_authentication_t  tls_auth_min;
 	getdns_eventloop_event  event;
-	getdns_tcp_state        tcp;
-	uint16_t                query_id;
 
 	int                     edns_maximum_udp_payload_size;
 	uint16_t                max_udp_payload_size;
@@ -234,10 +236,11 @@ typedef struct getdns_network_req
 	struct getdns_network_req *write_queue_tail;
 
 	/* Some fields to record info for return_call_reporting */
-	uint64_t                    debug_start_time;
-	uint64_t                    debug_end_time;
-	size_t                      debug_tls_auth_status;
-    size_t                      debug_udp;
+	uint64_t                debug_start_time;
+	uint64_t                debug_end_time;
+	getdns_auth_state_t     debug_tls_auth_status;
+	getdns_bindata          debug_tls_peer_cert;
+	size_t                  debug_udp;
 
 	/* When more space is needed for the wire_data response than is
 	 * available in wire_data[], it will be allocated seperately.
@@ -262,11 +265,13 @@ typedef struct getdns_network_req
 	uint8_t *response;
 	size_t   wire_data_sz;
 	uint8_t  wire_data[];
-
-
 	
 } getdns_network_req;
 
+static inline int _getdns_netreq_finished(getdns_network_req *req)
+{ return !req || (req->state & NET_REQ_FINISHED); }
+
+struct chain_head;
 /**
  * dns request - manages a number of network requests and
  * the initial data passed to getdns_general
@@ -279,35 +284,52 @@ typedef struct getdns_dns_req {
 	uint8_t name[256];
 	size_t  name_len;
 
-	getdns_append_name_t append_name;
-	const uint8_t *suffix;
-	size_t  suffix_len;
-	int suffix_appended;
-
 	uint16_t request_class;
 
-	/* canceled flag */
-	int canceled;
-
 	/* context that owns the request */
 	struct getdns_context *context;
 
+	getdns_append_name_t append_name;
+	const uint8_t *suffix;
+	size_t  suffix_len;
+	unsigned suffix_appended			: 1;
+
 	/* request extensions */
-	int dnssec_return_status;
-	int dnssec_return_only_secure;
-	int dnssec_return_validation_chain;
+	unsigned dnssec_return_status			: 1;
+	unsigned dnssec_return_only_secure		: 1;
+	unsigned dnssec_return_all_statuses		: 1;
+	unsigned dnssec_return_validation_chain		: 1;
+	unsigned dnssec_return_full_validation_chain	: 1;
 #ifdef DNSSEC_ROADBLOCK_AVOIDANCE
-	int dnssec_roadblock_avoidance;
-	int avoid_dnssec_roadblocks;
+	unsigned dnssec_roadblock_avoidance		: 1;
+	unsigned avoid_dnssec_roadblocks		: 1;
 #endif
-	int edns_cookies;
-	int edns_client_subnet_private;
-	uint16_t tls_query_padding_blocksize;
-	int return_call_reporting;
-	int add_warning_for_bad_dns;
+	unsigned edns_cookies				: 1;
+	unsigned edns_client_subnet_private		: 1;
+	unsigned return_call_reporting			: 1;
+	unsigned add_warning_for_bad_dns		: 1;
 
 	/* Internally used by return_validation_chain */
-	int dnssec_ok_checking_disabled;
+	unsigned dnssec_ok_checking_disabled		: 1;
+	unsigned is_sync_request			: 1;
+	unsigned is_dns_request				: 1;
+	unsigned request_timed_out			: 1;
+
+	/* The validating and freed variables are used to make sure a single
+	 * code path is followed while processing a DNS request, even when
+	 * callbacks are already fired whilst the registering/scheduling call
+	 * (i.e. ub_resolve_event) has not returned yet.
+	 *
+	 * validating is touched by _getdns_get_validation_chain only and
+	 * freed      is touched by _getdns_submit_netreq only
+	 */
+	unsigned validating				: 1;
+	int *freed;
+
+	/* Validation chain to be canceled when this request is canceled */
+	struct chain_head *chain;
+
+	uint16_t tls_query_padding_blocksize;
 
 	/* internally scheduled request */
 	internal_cb_t internal_cb;
@@ -322,6 +344,11 @@ typedef struct getdns_dns_req {
 	/* the transaction id */
 	getdns_transaction_t trans_id;
 
+	/* Absolute time (in milliseconds since epoch),
+	 * after which this dns request is expired; i.e. timed out
+	 */
+	uint64_t expires;
+
 	/* for scheduling timeouts when using libunbound */
 	getdns_eventloop_event timeout;
 
@@ -331,6 +358,13 @@ typedef struct getdns_dns_req {
 	/* Stuff for stub resolving */
 	struct getdns_upstreams *upstreams;
 
+	/* Linked list pointer for dns requests, for which answers are received
+	 * from open connections as aside-effect of doing a synchronous call.
+	 * See also the type definition of getdns_upstream in context.h for a
+	 * more elaborate description.
+	 */
+	struct getdns_dns_req *finished_next;
+
 	/* network requests for this dns request.
 	 * The array is terminated with NULL.
 	 *
@@ -385,7 +419,7 @@ extern getdns_dict *dnssec_ok_checking_disabled_avoid_roadblocks;
 
 /* dns request utils */
 getdns_dns_req *_getdns_dns_req_new(getdns_context *context, getdns_eventloop *loop,
-    const char *name, uint16_t request_type, getdns_dict *extensions);
+    const char *name, uint16_t request_type, getdns_dict *extensions, uint64_t *now_ms);
 
 void _getdns_dns_req_free(getdns_dns_req * req);
 
@@ -401,5 +435,7 @@ void _getdns_network_validate_tsig(getdns_network_req *req);
 
 void _getdns_netreq_reinit(getdns_network_req *netreq);
 
+const char * _getdns_auth_str(getdns_auth_state_t auth);
+
 #endif
 /* types-internal.h */
